<?php

class AIImagePro_Settings
{
    const OPTION_PREFERRED_IMAGE_EXTENSION = 'aiimagepro_setting_preferred_image_extension';
    const OPTION_MAX_NUMBER_OF_HISTORY_ITEMS = 'aiimagepro_setting_max_number_of_history_items';
    const OPTION_PREFERRED_AI_EDITOR = 'aiimagepro_setting_preferred_ai_editor';
    const OPTION_OPENAI_API_KEY = 'aiimagepro_setting_openai_api_key';
    const OPTION_STABILITY_AI_API_KEY = 'aiimagepro_setting_stability_ai_api_key';
    const OPTION_STABILITY_AI_DEFAULT_MODEL = 'aiimagepro_setting_stability_ai_default_model';
    const OPTION_STABILITY_AI_NUMBER_OF_STEPS = 'aiimagepro_setting_stability_ai_default_number_of_steps';
    const OPTION_STABILITY_AI_DEFAULT_SAMPLER = 'aiimagepro_setting_stability_ai_default_sampler';
    const OPTION_PLUGIN_VERSION = 'aiimagepro_plugin_version';

    // singleton instance
    private static $instance;

    // singleton constructor
    public static function instance()
    {
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function get_all_options () {
        return array (
            self::OPTION_PREFERRED_IMAGE_EXTENSION => 'png',
            self::OPTION_MAX_NUMBER_OF_HISTORY_ITEMS => 30,
            self::OPTION_PREFERRED_AI_EDITOR => 'dalle',
            self::OPTION_OPENAI_API_KEY => '',
            self::OPTION_STABILITY_AI_API_KEY => '',
            self::OPTION_STABILITY_AI_DEFAULT_MODEL => 'stable-diffusion-512-v2-1',
            self::OPTION_STABILITY_AI_NUMBER_OF_STEPS => 30,
            self::OPTION_STABILITY_AI_DEFAULT_SAMPLER => 'K_EULER_ANCESTRAL',
            self::OPTION_PLUGIN_VERSION => aiimagepro_get_plugin_version(),
        );
    }

    public function init_options()
    {
        $options = $this->get_all_options();
        foreach ($options as $option => $defaultValue) {
            if (get_option($option) === false) {
                $this->update_option($option, $defaultValue);
            }
        }
    }

    public function init_options_if_needed()
    {
        if (aiimagepro_get_plugin_version() !== $this->get_option(self::OPTION_PLUGIN_VERSION)) {
            $this->init_options();
        }
    }

    public function delete_options()
    {
        $options = $this->get_all_options();
        foreach ($options as $option => $defaultValue) {
            delete_option($option);
        }
    }

    public function get_option($option)
    {
        return get_option($option);
    }

    public function update_option($option, $value)
    {
        return update_option($option, $value);
    }

    public function render()
    {
        ?>
        <div class="wrap">
            <h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
            <form action="options.php" method="post">
                <?php
                // output security fields for the registered setting
                settings_fields( 'aiimagepro_options' );
                // output setting sections and their fields
                do_settings_sections( 'aiimagepro' );
                // output save settings button
                submit_button( esc_html__( 'Save Settings', 'ai-image-pro' ) );
                ?>
            </form>
        </div>
        <?php
    }

    public function register_settings()
    {
        add_settings_section(
            'aiimagepro_general_settings_section',
            esc_html__( 'General Settings', 'ai-image-pro' ),
            array ($this, 'aiimagepro_general_settings_section_callback'),
            'aiimagepro'
        );

        // Preferred image extension
        register_setting('aiimagepro_options', self::OPTION_PREFERRED_IMAGE_EXTENSION);

        add_settings_field(
            self::OPTION_PREFERRED_IMAGE_EXTENSION,
            esc_html__( 'Preferred image extension', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_preferred_image_extension_callback'),
            'aiimagepro',
            'aiimagepro_general_settings_section'
        );

        // Max number of history items
        register_setting('aiimagepro_options', self::OPTION_MAX_NUMBER_OF_HISTORY_ITEMS);

        add_settings_field(
            self::OPTION_MAX_NUMBER_OF_HISTORY_ITEMS,
            esc_html__( 'Maximum number of history items (undo/redo)', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_max_number_of_history_items_callback'),
            'aiimagepro',
            'aiimagepro_general_settings_section'
        );

        // preferred ai editor
        register_setting('aiimagepro_options', self::OPTION_PREFERRED_AI_EDITOR);

        add_settings_field(
            self::OPTION_PREFERRED_AI_EDITOR,
            esc_html__( 'Preferred AI Editor', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_preferred_ai_editor_callback'),
            'aiimagepro',
            'aiimagepro_general_settings_section'
        );

        $this->register_dalle_settings();
        $this->register_stability_ai_settings();
    }

    function register_dalle_settings () {
        add_settings_section(
            'aiimagepro_dalle_settings_section',
            esc_html__( 'OpenAI (Dall.E) Settings', 'ai-image-pro' ),
            array ($this, 'aiimagepro_dalle_settings_section_callback'),
            'aiimagepro'
        );

        // OpenAI API Key
        register_setting('aiimagepro_options', self::OPTION_OPENAI_API_KEY);

        add_settings_field(
            self::OPTION_OPENAI_API_KEY,
            esc_html__( 'OpenAI API key', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_openai_api_key_callback'),
            'aiimagepro',
            'aiimagepro_dalle_settings_section'
        );
    }

    function register_stability_ai_settings () {
        add_settings_section(
            'aiimagepro_stability_ai_settings_section',
            esc_html__( 'Stable Diffusion (Stability.ai) Settings', 'ai-image-pro' ),
            array ($this, 'aiimagepro_stability_ai_settings_section_callback'),
            'aiimagepro'
        );

        // Stability AI API Key
        register_setting('aiimagepro_options', self::OPTION_STABILITY_AI_API_KEY);

        add_settings_field(
            self::OPTION_STABILITY_AI_API_KEY,
            esc_html__( 'Stability.ai API key', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_stability_ai_api_key_callback'),
            'aiimagepro',
            'aiimagepro_stability_ai_settings_section'
        );

        // Default model
        register_setting('aiimagepro_options', self::OPTION_STABILITY_AI_DEFAULT_MODEL);

        add_settings_field(
            self::OPTION_STABILITY_AI_DEFAULT_MODEL,
            esc_html__( 'Default model', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_stability_ai_default_model_callback'),
            'aiimagepro',
            'aiimagepro_stability_ai_settings_section'
        );

        // Default number of steps
        register_setting('aiimagepro_options', self::OPTION_STABILITY_AI_NUMBER_OF_STEPS);

        add_settings_field(
            self::OPTION_STABILITY_AI_NUMBER_OF_STEPS,
            esc_html__( 'Default number of steps', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_stability_ai_default_number_of_steps_callback'),
            'aiimagepro',
            'aiimagepro_stability_ai_settings_section'
        );

        // Default sampler
        register_setting('aiimagepro_options', self::OPTION_STABILITY_AI_DEFAULT_SAMPLER);

        add_settings_field(
            self::OPTION_STABILITY_AI_DEFAULT_SAMPLER,
            esc_html__( 'Default sampler', 'ai-image-pro' ),
            array ($this, 'aiimagepro_setting_stability_ai_default_sampler_callback'),
            'aiimagepro',
            'aiimagepro_stability_ai_settings_section'
        );
    }

    function aiimagepro_general_settings_section_callback() {
        ?>
        <div class="ai-image-pro-settings-section">
            <div class="ai-image-pro-settings-section-button">
                <a href="https://youtu.be/qZgAJ-2ky9s" target="_blank" class="button button-primary">
                    &#9658;
                    <?php
                    echo esc_html__('Watch video tutorial', 'ai-image-pro');
                    ?>
                </a>
            </div>
            <div class="ai-image-pro-settings-section-content">
                <p>
                    <?php
                    echo esc_html__('For more information on how to configure and use AI Image Pro, please watch the video tutorial by clicking on the link on the right.', 'ai-image-pro');
                    ?>
                </p>
            </div>
        </div>
        <?php
    }

    function aiimagepro_setting_preferred_image_extension_callback() {
        $value = get_option(self::OPTION_PREFERRED_IMAGE_EXTENSION);
        ?>
        <select name="aiimagepro_setting_preferred_image_extension">
            <option selected value="png" <?php selected('png', $value, true); ?>>PNG</option>
            <option value="jpg" <?php selected('jpg', $value, true); ?>>JPG</option>
        </select>
        <p>
            <small>
                <?php
                echo esc_html__('Choose your preferred image extension in which images will be saved. PNG allows for transparency, but consumes a bit more space, while JPG doesn\'t allow for transparency but more space-efficient.', 'ai-image-pro');
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_setting_max_number_of_history_items_callback() {
        $value = get_option(self::OPTION_MAX_NUMBER_OF_HISTORY_ITEMS);
        ?>
        <input type="number" min="0" step="1" name="aiimagepro_setting_max_number_of_history_items" value="<?php echo $value; ?>" />
        <p>
            <small>
                <?php
                echo esc_html__('The maximum number of history items (undo/redo) that will be saved in the editor. Depending on your computer resources you might want to increase/decrease this amount as more steps will consume more resources in your web browser. Setting it to 0 will disable history.', 'ai-image-pro');
                ?>
            </small>
        <?php
    }

    function aiimagepro_setting_preferred_ai_editor_callback() {
        $value = get_option(self::OPTION_PREFERRED_AI_EDITOR);
        ?>
        <select name="aiimagepro_setting_preferred_ai_editor">
            <option selected value="dalle" <?php selected('ai', $value, true); ?>><?php echo esc_html__( 'Dall.E', 'ai-image-pro' ); ?></option>
            <option value="stability-ai" <?php selected('stability-ai', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion (Stability AI)', 'ai-image-pro' ); ?></option>
        </select>
        <p>
            <small>
                <?php
                echo esc_html__('Choose your preferred AI editor. Your preferred AI editor will appear first in the editor.', 'ai-image-pro');
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_dalle_settings_section_callback () {

    }

    function aiimagepro_stability_ai_settings_section_callback () {

    }

    function aiimagepro_setting_openai_api_key_callback () {
        $value = get_option(self::OPTION_OPENAI_API_KEY);
        ?>
        <input size="80" type="text" name="aiimagepro_setting_openai_api_key" value="<?php echo $value; ?>" />
        <p>
            <small>
                <?php
                echo esc_html__('Your OpenAI API key. You can get your API key by signing up for an account at ', 'ai-image-pro') . '<a href="https://platform.openai.com/signup">https://platform.openai.com/signup</a>';
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_setting_stability_ai_api_key_callback () {
        $value = get_option(self::OPTION_STABILITY_AI_API_KEY);
        ?>
        <input size="80" type="text" name="aiimagepro_setting_stability_ai_api_key" value="<?php echo $value; ?>" />
        <p>
            <small>
                <?php
                echo esc_html__('Your Stability.ai API key. You can get your API key by signing up for an account at ', 'ai-image-pro') . '<a href="https://beta.dreamstudio.ai/">https://beta.dreamstudio.ai/</a>';
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_setting_stability_ai_default_model_callback () {
        $value = get_option(self::OPTION_STABILITY_AI_DEFAULT_MODEL);
        ?>
        <select name="aiimagepro_setting_stability_ai_default_model">
            <option value="stable-diffusion-v1-5" <?php selected('stable-diffusion-v1-5', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion v1.5', 'ai-image-pro' ); ?></option>
            <option value="stable-diffusion-v1" <?php selected('stable-diffusion-v1', $value, true); ?> ><?php echo esc_html__( 'Stable Diffusion v1.4', 'ai-image-pro' ); ?></option>
            <option value="stable-diffusion-512-v2-0" <?php selected('stable-diffusion-512-v2-0', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion v2.0', 'ai-image-pro' ); ?></option>
            <option value="stable-diffusion-512-v2-1" <?php selected('stable-diffusion-512-v2-1', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion v2.1', 'ai-image-pro' ); ?></option>
            <option value="stable-diffusion-768-v2-1" <?php selected('stable-diffusion-768-v2-1', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion v2.1-768', 'ai-image-pro' ); ?></option>
            <option value="stable-diffusion-xl-beta-v2-2-2" <?php selected('stable-diffusion-xl-beta-v2-2-2', $value, true); ?>><?php echo esc_html__( 'Stable Diffusion v2.2.2-XL Beta', 'ai-image-pro' ); ?></option>
        </select>
        <p>
            <small>
                <?php
                echo esc_html__('The default model that will be used when generating an image using Stability.ai.', 'ai-image-pro');
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_setting_stability_ai_default_number_of_steps_callback () {
        $value = get_option(self::OPTION_STABILITY_AI_NUMBER_OF_STEPS);
        ?>
        <input type="number" step="1" min="10" max="150" name="aiimagepro_setting_stability_ai_default_number_of_steps" value="<?php echo $value; ?>" />
        <p>
            <small>
                <?php
                echo esc_html__('The default number of steps that will be used when generating an image using Stability.ai.', 'ai-image-pro');
                ?>
            </small>
        </p>
        <?php
    }

    function aiimagepro_setting_stability_ai_default_sampler_callback () {
        $value = get_option(self::OPTION_STABILITY_AI_DEFAULT_SAMPLER);
        ?>
        <select name="aiimagepro_setting_stability_ai_default_sampler">
            <option value="DDIM"><?php echo esc_html__( 'DDIM', 'ai-image-pro' ); ?></option>
            <option value="DDPM"><?php echo esc_html__( 'DDPM', 'ai-image-pro' ); ?></option>
            <option value="K_DPMPP_2M"><?php echo esc_html__( 'K_DPMPP_2M', 'ai-image-pro' ); ?></option>
            <option value="K_DPMPP_2S_ANCESTRAL"><?php echo esc_html__( 'K_DPMPP_2S_ANCESTRAL', 'ai-image-pro' ); ?></option>
            <option value="K_DPM_2"><?php echo esc_html__( 'K_DPM_2', 'ai-image-pro' ); ?></option>
            <option value="K_DPM_2_ANCESTRAL"><?php echo esc_html__( 'K_DPM_2_ANCESTRAL', 'ai-image-pro' ); ?></option>
            <option value="K_EULER"><?php echo esc_html__( 'K_EULER', 'ai-image-pro' ); ?></option>
            <option value="K_EULER_ANCESTRAL" selected><?php echo esc_html__( 'K_EULER_ANCESTRAL', 'ai-image-pro' ); ?></option>
            <option value="K_HEUN"><?php echo esc_html__( 'K_HEUN', 'ai-image-pro' ); ?></option>
            <option value="K_LMS"><?php echo esc_html__( 'K_LMS', 'ai-image-pro' ); ?></option>
        </select>
        <p>
            <small>
                <?php
                echo esc_html__('Choose your preferred sampler that will be used when generating images with Stability.ai', 'ai-image-pro');
                ?>
            </small>
        </p>
        <?php
    }

    public function admin_config_notice() {
        $this->max_upload_notice();
        $this->configure_plugin_notice();
    }

    private function max_upload_notice()
    {
        if (!isset($_GET['page']) || $_GET['page'] !== 'aiimagepro_image_editor_settings') {
            return;
        }

        $maxUploadSize = wp_max_upload_size();
        $maxUploadSize = $maxUploadSize / 1024 / 1024;
        $maxUploadSize = round($maxUploadSize, 2);

        if ($maxUploadSize < 20) {
            ?>
            <div id="ai-image-pro-notice" class="notice-warning notice is-dismissible">

                <div class="ai-image-pro-notice-txt">
                    <p>
                        <?php echo esc_html__('For the best experience, AI Image Pro requires a minimum upload size of 20MB. Please increase your upload size to avoid getting error when editing bigger images. Ask you hosting provider, or', 'ai-image-pro')?> <a href="https://www.wpbeginner.com/wp-tutorials/how-to-increase-the-maximum-file-upload-size-in-wordpress/" target="_blank"><?php echo esc_html__(' follow this guide ', 'ai-image-pro')?></a> <?php echo esc_html__('for more information.', 'ai-image-pro')?>
                    </p>
                </div>
            </div>
            <?php
        }
    }

    private function configure_plugin_notice()
    {
        global $pagenow;

        if ($pagenow !== 'plugins.php') {
            return;
        }

        // if neither openai not stability ai keys are set, show the notice
        $openaiKey = $this->get_option(self::OPTION_OPENAI_API_KEY);
        $stabilityAiKey = $this->get_option(self::OPTION_STABILITY_AI_API_KEY);

        if ( empty($openaiKey) && empty($stabilityAiKey) ) {
            ?>
            <div id="ai-image-pro-notice" class="updated notice is-dismissible">

                <div class="ai-image-pro-notice-txt">
                    <p>
                        <?php echo esc_html__('Thank you for purchasing AI Image Pro! Please consider entering your AI API keys in order to start leveraging AI image generation.', 'ai-image-pro')?>
                    </p>
                </div>
                <div class="ai-image-pro-btn-container">
                    <a href="<?php echo admin_url( 'admin.php?page=aiimagepro_image_editor_settings' ); ?>" id="ai-image-pro-btn"><?php echo esc_html__('Configure AI Image Pro', 'ai-image-pro')?></a>
                </div>
            </div>
            <?php
        }
    }
}
